# Model recovery simulation for Binary ROC #
# Authors: David Kellen and Karl Christoph Klauer #
# date: 1-7-2010 #

################################
# STEP 0: Preliminary Settings #
################################
rm(list=ls())   #clean console
np=1000         #number of participants simulated
NumPoints=3		# number of points in ROC
N=304           #number of items (per item type)
fc=40            #individual fit cycle

###################################
# STEP 1: Sample Model Parameters #
###################################

mQSDT=matrix(NA,ncol=2+NumPoints,nrow=np)            #matrix that stores the SDT simulated parameter values   
mQMPT=matrix(NA,ncol=2+NumPoints,nrow=np)            #matrix that stores the MPT simulated parameter values   

for (i in 1:np){

    #############
    #=== SDT ===#
    #############
		
		#####== Unrestricted criterion variability ==#####
			#QSDT<-c(runif(5,min=c(.5,0.5,0,rep(0.01,(NumPoints-1))),max=c(2,4,3,rep(1,(NumPoints-1)))))
		
		#####== Small criterion variability ==#####
			QSDT<-c(runif(5,min=c(.5,0.5,0,rep(0.01,(NumPoints-1))),max=c(2,4,3,rep(0.1,(NumPoints-1)))))
			
		
		###################################
		#==  Define response criterion  ==#
		###################################
				
		s=QSDT[1]	;	mu=QSDT[2]	;	C=(QSDT[-(1:2)])
				
		for (j in 1:NumPoints){
			if (j==1)C[j]=C[j]
			else C[j]=C[j-1]-C[j]
			}
		mQSDT[i,]=c(s,mu,C)
	
    #############
    #=== MPT ===#
    #############
		
		##########################################
		#== Unrestricted criterion variability ==#
		##########################################
		
			#QMPT<-c(runif(10,min=c(.01,.01,rep(.01,5)),max=c(.99,.99,rep(.99,5))))  #sample parameters at random
		
			#a<-c(rep(0,NumPoints))											#defines vector for response bias
			#for (j in 1:NumPoints) a[j]<-QMPT[2+j]							# ascribes sampled values for the bias parameters 
			#a<-sort(a)														#orders the parameters
			#mQMPT[i,]<-c(QMPT[1:2],a)
		
		###################################
		#== Small criterion variability ==#
		###################################
		
			QMPT<-c(runif(10,min=c(.01,.01,rep(.01,5)),max=c(.99,.99,rep(.99,5))))  #sample random MPT parameters.
			
			a<-c(rep(0,NumPoints))											#defines vector for response bias
			for (j in 1:NumPoints) a[j]<-QMPT[2+j]							# ascribes sampled values for the bias parameters 
			a<-sort(a)														#orders the parameters
			
			a[1]<-a[2]+(a[1]-a[2])*0.1	;	a[3]<-a[2]+(a[3]-a[2])*0.1		#produces the small criterion variability
			mQMPT[i,]<-c(QMPT[1:2],a)										#puts everything together inside the MPT parameter matrix
		
}
   
###################################
#     STEP 2: Generate Data       #
###################################

gdatSDT=matrix(0,nrow=np,ncol=(4*NumPoints))        #matrix that stores the SDT generated data
gdatMPT=matrix(0,nrow=np,ncol=(4*NumPoints))        #matrix that stores the MPT generated data

for (i in 1:np){

    #############
    #=== SDT ===#
    #############

	mu=mQSDT[i,2]	;	s=mQSDT[i,1]	;	c=c(mQSDT[1,3:(2+NumPoints)])
	
	for (j in 1:NumPoints){
		gdatSDT[i,(j-1)*4+1]=rbinom(1,N,(1-pnorm((c[j]-mu)*s)))
		gdatSDT[i,(j-1)*4+2]=N-gdatSDT[i,(j-1)*4+1]
		gdatSDT[i,(j-1)*4+3]=rbinom(1,N,(1-pnorm(c[j])))
		gdatSDT[i,(j-1)*4+4]=N-gdatSDT[i,(j-1)*4+3]
	}

  
    #############
    #=== MPT ===#
    #############

	rv=mQMPT[i,1]	;	ri=mQMPT[i,2]	;	a=c(mQMPT[1,3:(2+NumPoints)])
	
	for (j in 1:NumPoints){
		gdatMPT[i,(j-1)*4+1]=rbinom(1,N,(rv+(1-rv)*a[j]))
		gdatMPT[i,(j-1)*4+2]=N-gdatMPT[i,(j-1)*4+1]
		gdatMPT[i,(j-1)*4+3]=rbinom(1,N,((1-ri)*a[j]))
		gdatMPT[i,(j-1)*4+4]=N-gdatMPT[i,(j-1)*4+3]
	}
}

###########################################
#   STEP 3: Define models for fitting     #
###########################################   
   
    #############
    #=== SDT ===#
    #############
   

	gSDT<-function(Q){ 
		mu<-Q[1]
		s<-Q[2]
		c1<-Q[3] #conservative criterion
		c2<-c1-Q[4] #shift for next criterion
		c3<-c2-Q[5]
		e<-c(rep(0,12))
		
		e[1]<-  N*(1-pnorm((c1-mu)*s))
		e[2]<-  N-e[1]
		e[3]<-  N*(1-pnorm(c1))
		e[4]<-  N-e[3]

		e[5]<-  N*(1-pnorm((c2-mu)*s))
		e[6]<-  N-e[5]                          
		e[7]<-  N*(1-pnorm(c2))
		e[8]<-  N-e[7]                         

		e[9]<-  N*(1-pnorm((c3-mu)*s))
		e[10]<- N-e[9]
		e[11]<- N*(1-pnorm(c3))
		e[12]<- N-e[11]

           
	tempsum<-0
	for (i in 1:12) {
		if (d[i]==0) t<-0.0
		else t<- 2* d[i]*(log(d[i])-log(e[i]))
		tempsum<-tempsum+t
	}
	tempsum
}
   
    #############
    #=== MPT ===#
    #############
	
	
	gMPT<-function(Q){ 
		rv<-Q[1]
		ri<-Q[2]
		a3<-Q[5]	#this is the most liberal criterion
		a2<-Q[4]*a3	
		a1<-Q[3]*a2
		
		e<-c(rep(0,12))
		
		e[1]=N*(rv+(1-rv)*a1)
		e[2]=N-e[1]
		e[3]=N*((1-ri)*a1)
		e[4]=N-e[3]

		e[5]=N*(rv+(1-rv)*a2)
		e[6]=N-e[5]
		e[7]=N*((1-ri)*a2)
		e[8]=N-e[7]                       

		e[9]=N*(rv+(1-rv)*a3)
		e[10]=N-e[9]
		e[11]=N*((1-ri)*a3)
		e[12]=N-e[11]

           
	tempsum<-0
	for (i in 1:12) {
		if (d[i]==0) t<-0.0
		else t<- 2* d[i]*(log(d[i])-log(e[i]))
		tempsum<-tempsum+t
	}
	tempsum
}
	
	

###################################
#     STEP 4: Cross-fit models    #
###################################

								#separate vectors that store goodness of fit values for each case (Model_Data)
Fit_SDT_SDT=c(rep(NA,np))			#SDT model fitted to SDT data
Fit_SDT_MPT=c(rep(NA,np))			#SDT model fitted to MPT data
Fit_MPT_SDT=c(rep(NA,np))			#MPT model fitted to SDT data
Fit_MPT_MPT=c(rep(NA,np))			#MPT model fitted to MPT data
	
	
	####################################
	#=== FIT SDT  MODEL TO SDT DATA ===#
	####################################
	
	for (i in 1:np){
		
		d=gdatSDT[i,]
		bestSDT<-Inf

	for (m in 1:fc) {
		fitSDT<-nlminb(c(runif(5)),gSDT,control=list(eval.max=1500,iter.max=1500),lower=c(0,0,-Inf,rep(0,2)),upper=c(rep(Inf,5)))
		if(fitSDT$objective<bestSDT)  {bestSDT<-fitSDT$objective;Fit_SDT_SDT[i]<-bestSDT}
	}
	if (bestSDT>20) {
	for (m in 1:fc) {
		fitSDT<-nlminb(c(runif(5,min=c(.5,0,0,rep(0,2)),max=c(2,4,3,rep(1,2)))),gSDT,control=list(eval.max=1500,iter.max=1500),lower=c(0.1,0,-5,rep(0,2)),upper=c(rep(Inf,5)))
		if(fitSDT$objective<bestSDT)  {bestSDT<-fitSDT$objective;Fit_SDT_SDT[i]<-bestSDT}
	}
	}		
	print(i);print(Fit_SDT_SDT[i])
	}
 	

	####################################
	#=== FIT SDT  MODEL TO MPT DATA ===#
	####################################
	
	for (i in 1:np){
		
		d=gdatMPT[i,]
		bestSDT<-Inf
		
	for (m in 1:fc) {
		fitSDT<-nlminb(c(runif(5)),gSDT,control=list(eval.max=1500,iter.max=1500),lower=c(0,0,-Inf,rep(0,2)),upper=c(rep(Inf,5)))
		if(fitSDT$objective<bestSDT)  {bestSDT<-fitSDT$objective;Fit_SDT_MPT[i]<-bestSDT}
	}
	if (bestSDT>20) {
	for (m in 1:fc) {
		fitSDT<-nlminb(c(runif(5,min=c(.5,0,0,rep(0,2)),max=c(2,4,3,rep(1,2)))),gSDT,control=list(eval.max=1500,iter.max=1500),lower=c(0.1,0,-5,rep(0,2)),upper=c(rep(Inf,5)))
		if(fitSDT$objective<bestSDT)  {bestSDT<-fitSDT$objective;Fit_SDT_MPT[i]<-bestSDT}
	}
	}
	print(i);print(Fit_SDT_MPT[i])
	}
	
	###################################
	#=== FIT MPT MODEL TO SDT DATA ===#
	###################################
	
	for (i in 1:np){
	
	d=gdatSDT[i,]
	
	bestMPT<-Inf
		
	for (m in 1:fc) {
	fitMPT<-nlminb(runif(5),gMPT,control=list(eval.max=1500,iter.max=1500),lower=c(rep(0,5)),upper=c(rep(1,5)))
	if(fitMPT$objective<bestMPT)  {bestMPT<-fitMPT$objective;Fit_MPT_SDT[i]<-bestMPT }
	}
	if (bestMPT>20) {
	for (m in 1:fc) {
	fitMPT<-nlminb(c(runif(5)),gMPT,control=list(eval.max=1500,iter.max=1500),lower=c(rep(0,5)),upper=c(rep(1,5)))
	if(fitMPT$objective<bestMPT)  {bestMPT<-fitMPT$objective;Fit_MPT_SDT[i]<-bestMPT }
	}
	}		
	print(i);print(Fit_MPT_SDT[i])
	}
	
	###################################
	#=== FIT MPT MODEL TO MPT DATA ===#
	###################################
	
	for (i in 1:np){
	
	d=gdatMPT[i,]
	
	bestMPT<-Inf
	for (m in 1:fc) {
	fitMPT<-nlminb(c(runif(5)),gMPT,control=list(eval.max=1500,iter.max=1500),lower=c(rep(0,5)),upper=c(rep(1,5)))
	if(fitMPT$objective<bestMPT)  {bestMPT<-fitMPT$objective;Fit_MPT_MPT[i]<-bestMPT }
	}
	if (bestMPT>20) {
	for (m in 1:fc) {
	fitMPT<-nlminb(c(runif(5)),gMPT,control=list(eval.max=1500,iter.max=1500),lower=c(rep(0,5)),upper=c(rep(1,5)))
	if(fitMPT$objective<bestMPT)  {bestMPT<-fitMPT$objective;Fit_MPT_MPT[i]<-bestMPT }
	}
	}	
	print(i);print(Fit_MPT_MPT[i])
	}


##############################################
#   STEP 5: goodness-of-fit model selection  #
##############################################

overfitSDT=length(Fit_MPT_MPT[Fit_MPT_MPT>Fit_SDT_MPT])			# frequency of SDT providing better goodness of fit for MPT data
overfitSDT
 
overfitMPT=length(Fit_SDT_SDT[Fit_SDT_SDT>Fit_MPT_SDT])			# frequency of MPT providing better goodness of fit for SDT data 
overfitMPT

overfitMPT-overfitSDT  #positive value means MPT overfits more


# ... fertig!!!




