# Reconstruction of Dube code: model recovery simulation for 5-point ratings ROC #
# Author: David Kellen #
# Date: 25-06-2010 #

################################
# STEP 0: Preliminary Settings #
################################
rm(list=ls())	#clean console
np=500			#number of participants simulated
N=304			#number of items (per item type)
kSDT=14			#number of parameters of SDT
kMPT=16			#number of parameters of MPT
fc=20             	#individual fit cycle

###################################
# STEP 1: Sample Model Parameters #
###################################

mQSDT=matrix(0,nrow=np, ncol=14)
mQMPT=matrix(0,nrow=np, ncol=19)		#this matrix includes redundant parameters (thats why its size is 19 and not 16)

for (i in 1:np){
	
	#############
	#=== SDT ===#
	#############
	
	##QSDT<-c(runif(14,min=c(0.5,0,0.5,0,0,rep(0,4),0,rep(0,4)),max=c(3,5,3,5,7,rep(2,4),7,rep(2,4))))		#the rep(x,y) parts specify how far apart the criteria can be
	
	QSDT<-c(runif(14,min=c(.5,0.5,.5,0.5,0,rep(.01,4),0.5,rep(.01,4)),max=c(2,3.5,2,3.5,3,rep(.5,4),3,rep(.5,4)))) # this is DRH version of parameter sampling
	
	
	c1b<-QSDT[5] ; c2b<-c1b-QSDT[6] ; c3b<-c2b-QSDT[7] ; c4b<-c3b-QSDT[8] ; c5b<-c4b-QSDT[9]		#cumulative shifts from conservative criterion (believable items)
	c1u<-QSDT[10] ; c2u<-c1u-QSDT[11] ; c3u<-c2u-QSDT[12] ; c4u<-c3u-QSDT[13] ; c5u<-c4u-QSDT[14]	#cumulative shifts from conservative criterion (unbelievable items)
	cb=c(c1b,c2b,c3b,c4b,c5b)
	cu=c(c1u,c2u,c3u,c4u,c5u)
	mQSDT[i,]=c(QSDT[1:4],cb,cu)			#puts everything together inside the SDT parameter matrix
	
	#############
	#=== MPT ===#
	#############
	
		#####################
		#====DRH version====#
		#####################
	
#			QMPT<-c(runif(16,min=c(rep(0,16)),max=c(rep(1,4),rep(50000,12))))			#upper value 50000 because runif won't sample from infinite range. This parameter sampling is equal to DRH
#		
#			pmake6<-function(x) {										# function that produces response mapping parameters (6 point mapping)
#				temp<-c(x[1:2],1,x[3:5])
#				xsum<-sum(temp[1:6])
#				temp<-temp/xsum
#				temp
#			}
#	
#			pmake3<-function(x) {										# function that produces response mapping parameters (3 point mapping)
#				temp<-c(x[1:2],1)
#				xsum<-sum(temp[1:3])
#				temp<-temp/xsum
#				temp
#			}
#	
#			srm=pmake3(QMPT[5:6])					#detect-state response mapping (sh,sm, and sl)
#			bam=pmake6(QMPT[7:11])					#believeable items guessing response mapping (ab)
#			uam=pmake6(QMPT[12:16])					#believeable items guessing response mapping (au)
#			mQMPT[i,]=c(QMPT[1:4],srm,bam,uam)		#puts everything together inside the MPT parameter matrix
	
#}


		#####################
		#====K&K version====#
		#####################
		
		
 			QMPT<-c(runif(19,min=c(rep(0.001,19)),max=c(rep(0.999,4),rep(50000,15))))			#upper value 100000 because runif won't sample from infinite range. This parameter sampling is equal to DRH
			
 			pmake<-function(x){
 				temp<-c(x)
 				xsum<-sum(x)
 				temp<-temp/xsum
 				temp
 			}
			
 			srm=pmake(QMPT[5:7])					#detect-state response mapping (sh,sm, and sl)
 			bam=pmake(QMPT[8:13])					#believeable items guessing response mapping (ab)
 			uam=pmake(QMPT[14:19])					#unbelieveable items guessing response mapping (au)
 			mQMPT[i,]=c(QMPT[1:4],srm,bam,uam)		#puts everything together inside the MPT parameter matrix

}

###################################
#     STEP 2: Generate Data	      #
###################################

gdatSDT=matrix(0,nrow=np,ncol=(24))		#matrix that stores the SDT generated data
gdatMPT=matrix(0,nrow=np,ncol=(24))		#matrix that stores the MPT generated data

for (i in 1:np){

	#############
	#=== SDT ===#
	#############
	
	bs<-mQSDT[i,1];bd<-mQSDT[i,2];su<-mQSDT[i,3];du<-mQSDT[i,4]									#means and standard deviations of signal distribution
	c1b<-mQSDT[i,5];c2b<-mQSDT[i,6];c3b<-mQSDT[i,7];c4b<-mQSDT[i,8];c5b<-mQSDT[i,9]				#response criteria for believable items
	c1u<-mQSDT[i,10];c2u<-mQSDT[i,11];c3u<-mQSDT[i,12];c4u<-mQSDT[i,13];c5u<-mQSDT[i,14]		#response criteria for unbelievable items
	
	
	
											#produces expected rating probabilities for syllogisms  
											#the order of the rating goes from most conservative (sure valid) to most liberal (sure invalid)
		#valid/believeable									
	sdt_vb=c((pnorm((bd-c1b)*bs)) , (pnorm((bd-c2b)*bs)-pnorm((bd-c1b)*bs)) , (pnorm((bd-c3b)*bs)-pnorm((bd-c2b)*bs)),
			(pnorm((bd-c4b)*bs)-pnorm((bd-c3b)*bs)) , (pnorm((bd-c5b)*bs)-pnorm((bd-c4b)*bs)) , (1-pnorm((bd-c5b)*bs)))

	#invalid/believeable
	sdt_ib=c(pnorm(-c1b) , (pnorm(-c2b)-pnorm(-c1b)) , (pnorm(-c3b)-pnorm(-c2b)) , (pnorm(-c4b)-pnorm(-c3b)) , (pnorm(-c5b)-pnorm(-c4b)) , (1-pnorm(-c5b)))
	
	#valid/unbelieveable
	sdt_vu=c(pnorm((du-c1u)*su) , (pnorm((du-c2u)*su)-pnorm((du-c1u)*su)) , (pnorm((du-c3u)*su)-pnorm((du-c2u)*su)),
			(pnorm((du-c4u)*su)-pnorm((du-c3u)*su)) , (pnorm((du-c5u)*su)-pnorm((du-c4u)*su)) , (1-pnorm((du-c5u)*su)))
	
	#invalid/unbelieveable
	sdt_iu=c(pnorm(-c1u) , (pnorm(-c2u)-pnorm(-c1u)) , (pnorm(-c3u)-pnorm(-c2u)) , (pnorm(-c4u)-pnorm(-c3u)) , (pnorm(-c5u)-pnorm(-c4u)) , (1-pnorm(-c5u)))

	
	gdatSDT[i,1:6]=rmultinom(1, N,sdt_vb)				#simulates response frequencies according to the expected probabilities (multinomial distribution)
	gdatSDT[i,7:12]=rmultinom(1, N,sdt_ib)
	gdatSDT[i,13:18]=rmultinom(1, N,sdt_vu)
	gdatSDT[i,19:24]=rmultinom(1, N,sdt_iu)


	#############
	#=== MPT ===#
	#############
	
	
	rvb<-mQMPT[i,1] ; rib<-mQMPT[i,2] ; rvu<-mQMPT[i,3] ; riu<-mQMPT[i,4] 												#detection parameters
	sh<-mQMPT[i,5] ; sm<-mQMPT[i,6] ; sl<-mQMPT[i,7]																	#detection response mapping
	ab1<-mQMPT[i,8] ; ab2<-mQMPT[i,9] ; ab3<-mQMPT[i,10] ; ab4<-mQMPT[i,11] ; ab5<-mQMPT[i,12] ; ab6<-mQMPT[i,13] 		#guessing response mapping for believable items
	au1<-mQMPT[i,14] ; au2<-mQMPT[i,15] ; au3<-mQMPT[i,16] ; au4<-mQMPT[i,17] ; au5<-mQMPT[i,18] ; au6<-mQMPT[i,19] 	#guessing response mapping for unbelievable items

	#valid/believeable
	mpt_vb=c((rvb*sh+(1-rvb)*ab1) , (rvb*sm+(1-rvb)*ab2) , (rvb*sl+(1-rvb)*ab3),
		((1-rvb)*ab4) , ((1-rvb)*ab5) , ((1-rvb)*ab6))

	#invalid/believeable
	mpt_ib=c(((1-rib)*ab1) , ((1-rib)*ab2) , ((1-rib)*ab3),
		(rib*sl+(1-rib)*ab4) , (rib*sm+(1-rib)*ab5) , (rib*sh+(1-rib)*ab6))                    

	#valid/unbelieveable
	mpt_vu=c((rvu*sh+(1-rvu)*au1) , (rvu*sm+(1-rvu)*au2) , (rvu*sl+(1-rvu)*au3),
		((1-rvu)*au4) , ((1-rvu)*au5) , ((1-rvu)*au6))

	#invalid/unbelieveable
	mpt_iu=c(((1-riu)*au1) , ((1-riu)*au2) , ((1-riu)*au3),
		(riu*sl+(1-riu)*au4) , (riu*sm+(1-riu)*au5) , (riu*sh+(1-riu)*au6))    


	gdatMPT[i,1:6]=rmultinom(1, N,mpt_vb)
	gdatMPT[i,7:12]=rmultinom(1, N,mpt_ib)
	gdatMPT[i,13:18]=rmultinom(1, N,mpt_vu)
	gdatMPT[i,19:24]=rmultinom(1, N,mpt_iu)
	
	}

###########################################
#   STEP 3: Define models for fitting     #
###########################################
	

	########
	# SDT  #
	########
	
	gSDT<-function(mQSDT){ 
	
		bs<-mQSDT[1] ; bd<-mQSDT[2] ; su<-mQSDT[3] ; du<-mQSDT[4]											# standard deviation and means of signal ditributions
		
		c1b<-mQSDT[5] ; c2b<-c1b-mQSDT[6] ; c3b<-c2b-mQSDT[7] ; c4b<-c3b-mQSDT[8] ; c5b<-c4b-mQSDT[9]		#response criteria (starts at more conservative and shifts to more liberal)
		c1u<-mQSDT[10] ; c2u<-c1u-mQSDT[11] ; c3u<-c2u-mQSDT[12] ; c4u<-c3u-mQSDT[13] ; c5u<-c4u-mQSDT[14]
	
		e<-c(rep(0,24))	
		
		#valid/believeable									
	
		e[1]=N*(pnorm((bd-c1b)*bs)) 
		e[2]=N*(pnorm((bd-c2b)*bs)-pnorm((bd-c1b)*bs)) 
		e[3]=N*(pnorm((bd-c3b)*bs)-pnorm((bd-c2b)*bs))
		e[4]=N*(pnorm((bd-c4b)*bs)-pnorm((bd-c3b)*bs)) 
		e[5]=N*(pnorm((bd-c5b)*bs)-pnorm((bd-c4b)*bs)) 
		e[6]=N*(1-pnorm((bd-c5b)*bs))

		#invalid/believeable
	
		e[7]=N*pnorm(-c1b) 
		e[8]=N*(pnorm(-c2b)-pnorm(-c1b)) 
		e[9]=N*(pnorm(-c3b)-pnorm(-c2b)) 
		e[10]=N*(pnorm(-c4b)-pnorm(-c3b)) 
		e[11]=N*(pnorm(-c5b)-pnorm(-c4b)) 
		e[12]=N*(1-pnorm(-c5b))
	
		#valid/unbelieveable
	
		e[13]=N*(pnorm((du-c1u)*su)) 
		e[14]=N*(pnorm((du-c2u)*su)-pnorm((du-c1u)*su)) 
		e[15]=N*(pnorm((du-c3u)*su)-pnorm((du-c2u)*su))
		e[16]=N*(pnorm((du-c4u)*su)-pnorm((du-c3u)*su)) 
		e[17]=N*(pnorm((du-c5u)*su)-pnorm((du-c4u)*su)) 
		e[18]=N*(1-pnorm((du-c5u)*su))
	
		#invalid/unbelieveable
	
		e[19]=N*pnorm(-c1u) 
		e[20]=N*(pnorm(-c2u)-pnorm(-c1u))
		e[21]=N*(pnorm(-c3u)-pnorm(-c2u))
		e[22]=N*(pnorm(-c4u)-pnorm(-c3u))
		e[23]=N*(pnorm(-c5u)-pnorm(-c4u))
		e[24]=N*(1-pnorm(-c5u))	
	
		tempsum<-0
		for (i in 1:24) {
			if (d[i]==0) t<-0.0
			else t<- 2* d[i]*(log(d[i])-log(e[i]))
			tempsum<-tempsum+t
			# print(e)
			}
			tempsum
	}
	
		
	########
	# MPT  #
	########
	
	
	pmake<- function(x,m) {
		temp <- c(x,1.0)
		xsum<-sum(temp[1:m])
		temp<-temp/xsum
		temp
		#print(xsum)
		}
		##Q1-4: rvb, rib,ivu,riu
		##Q5-9: 5 non-negative numbers generating bias parameters ab
		##Q10-14: 5 non-negative numbers generating bias parameters au
		##Q15-16: 2 non-negative numbers generating sh,sm, and sl

	gMPT<-function(Q){ 
		rvb <- Q[1]
		rib <- Q[2]
		rvu <- Q[3]
		riu <- Q[4]
		ab<-pmake(Q[5:9],6)
		au<-pmake(Q[10:14],6)
		s <- pmake(Q[15:16],3)
		sh<-s[1]
		sm<-s[2]
		sl<-s[3]

		e<-c(rep(0,24))
		
		#valid/believeable
		
		e[1]<-  rvb*sh
		e[2]<-  rvb*sm
		e[3]<-  rvb*sl
		for (i in 1:6) e[i]<-N*(e[i]+(1-rvb)*ab[i])

		#valid/unbelieveable
		
		e[10]<- rib*sl
		e[11]<- rib*sm                            
		e[12]<- rib*sh                    
		for (i in 7:12) e[i]<-N*(e[i]+(1-rib)*ab[i-6])

		#valid/unbelieveable
		
		e[13]<-  rvu*sh
		e[14]<-  rvu*sm
		e[15]<-  rvu*sl
		for (i in 13:18) e[i]<-N*(e[i]+(1-rvu)*au[i-12])

		#invalid/unbelieveable
		
		e[22]<- riu*sl
		e[23]<- riu*sm                            
		e[24]<- riu*sh                    
		for (i in 19:24) e[i]<-N*(e[i]+(1-riu)*au[i-18])

		tempsum<-0
		for (i in 1:24) {
			if (d[i]==0) t<-0.0
			else t<- 2* d[i]*(log(d[i])-log(e[i]))
		tempsum<-tempsum+t
		# print(e)
		}
		tempsum
	}

###################################
#     STEP 4: Cross-fit models    #
###################################

								#separate vectors that store goodness of fit values for each case (Model_Data)
Fit_SDT_SDT=c(rep(NA,np))			#SDT model fitted to SDT data
Fit_SDT_MPT=c(rep(NA,np))			#SDT model fitted to MPT data
Fit_MPT_SDT=c(rep(NA,np))			#MPT model fitted to SDT data
Fit_MPT_MPT=c(rep(NA,np))			#MPT model fitted to MPT data
	
	####################################
	#=== FIT SDT  MODEL TO SDT DATA ===#
	####################################
	
	for (i in 1:np){
		
		d=gdatSDT[i,]
		bestSDT<-Inf
		for (m in 1:fc) {
			fitSDT<-nlminb(runif(14,min=0.15),gSDT,control=list(eval.max=1500,iter.max=1500),lower=c(0,0,0,0,-Inf,rep(0,4),-Inf,rep(0,4)),upper=c(rep(Inf,14)))		# K&K version
			#fitSDT<-nlminb(runif(14,min=0.15),gSDT,control=list(eval.max=1500,iter.max=1500),lower=c(.1,0,.1,0,-1,rep(0.01,4),-1,rep(.01,4)),upper=c(2,4,2,4,4,rep(2,4),4,rep(2,4)))	# DRH version
			if(fitSDT$objective<bestSDT)  {bestSDT<-fitSDT$objective;Fit_SDT_SDT[i]<-bestSDT}
		}		
	print(i);print(Fit_SDT_SDT[i])
	}
	

	####################################
	#=== FIT SDT  MODEL TO MPT DATA ===#
	####################################
	
	for (i in 1:np){
		
		d=gdatMPT[i,]
		bestSDT<-Inf
		for (m in 1:fc) {
			fitSDT<-nlminb(runif(14,min=0.15),gSDT,control=list(eval.max=1500,iter.max=1500),lower=c(0,0,0,0,-Inf,rep(0,4),-Inf,rep(0,4)),upper=c(rep(Inf,14)))		# K&K version
			#fitSDT<-nlminb(runif(14,min=0.15),gSDT,control=list(eval.max=1500,iter.max=1500),lower=c(.1,0,.1,0,-1,rep(0.01,4),-1,rep(.01,4)),upper=c(2,4,2,4,4,rep(2,4),4,rep(2,4)))	# DRH version
			if(fitSDT$objective<bestSDT)  {bestSDT<-fitSDT$objective;Fit_SDT_MPT[i]<-bestSDT}
		}		
	print(i);print(Fit_SDT_MPT[i])
	}
	
	###################################
	#=== FIT MPT MODEL TO SDT DATA ===#
	###################################
	
	for (i in 1:np){
	
		d=gdatSDT[i,]
		bestMPT<-Inf
		for (m in 1:fc) {
			fitMPT<-nlminb(runif(16),gMPT,control = list(eval.max=1500,iter.max=1500),lower=c(rep(0,16)),upper=c(rep(1,4),rep(Inf,12)))
			if (fitMPT$objective <bestMPT) {bestMPT<-fitMPT$objective; Fit_MPT_SDT[i]<-bestMPT}
		}
	print(i);print(Fit_MPT_SDT[i])
	}
	
	###################################
	#=== FIT MPT MODEL TO MPT DATA ===#
	###################################
	
	for (i in 1:np){
	
		d=gdatMPT[i,]
		bestMPT<-Inf
		for (m in 1:fc) {
			fitMPT<-nlminb(runif(16),gMPT,control = list(eval.max=1500,iter.max=1500),lower=c(rep(0,16)),upper=c(rep(1,4),rep(Inf,12)))
			if (fitMPT$objective <bestMPT) {bestMPT<-fitMPT$objective; Fit_MPT_MPT[i]<-bestMPT}
		}
	print(i);print(Fit_MPT_MPT[i])
	}

	
##############################################
#     STEP 5: AIC and BIC model selection    #
##############################################

sdtAIC=c(rep(2*14,np)) ; sdtBIC=c(rep(log(1216)*14,np))			# calculates AIC and BIC punishment factors for the SDT model
mptAIC=c(rep(2*16,np)) ; mptBIC=c(rep(log(1216)*16,np))			# calculates AIC and BIC punishment factors for the MPT model

#===SDT model selection values===#
	AIC_SDT_SDT=Fit_SDT_SDT+sdtAIC
	AIC_SDT_MPT=Fit_SDT_MPT+sdtAIC
	BIC_SDT_SDT=Fit_SDT_SDT+sdtBIC
	BIC_SDT_MPT=Fit_SDT_MPT+sdtBIC


#===MPT model selection values===#
	AIC_MPT_SDT=Fit_MPT_SDT+mptAIC
	AIC_MPT_MPT=Fit_MPT_MPT+mptAIC
	BIC_MPT_SDT=Fit_MPT_SDT+mptBIC
	BIC_MPT_MPT=Fit_MPT_MPT+mptBIC


overfitMPT_AIC=length(AIC_SDT_SDT[AIC_SDT_SDT>AIC_MPT_SDT])			# frequency of MPT providing better AIC for SDT data 
overfitMPT_BIC=length(BIC_SDT_SDT[BIC_SDT_SDT>BIC_MPT_SDT])			# frequency of MPT providing better BIC for SDT data
overfitMPT_AIC ; overfitMPT_BIC

overfitSDT_AIC=length(AIC_MPT_MPT[AIC_MPT_MPT>AIC_SDT_MPT])			# frequency of SDT providing better AIC for MPT data 
overfitSDT_BIC=length(BIC_MPT_MPT[BIC_MPT_MPT>BIC_SDT_MPT])			# frequency of SDT providing better BIC for MPT data
overfitSDT_AIC ; overfitSDT_BIC

overfitMPT_AIC-overfitSDT_AIC  #positive value means MPT overfits more, even with AIC
overfitMPT_BIC-overfitSDT_BIC  #positive value means MPT overfits more, even with BIC


# ... fertig!!!
